@@ -1,9 +1,17 @@
 """Minimalist implementation of Single Transferable Vote."""
 import collections
+import itertools
 
 import numpy as np
 
-RoundResult = collections.namedtuple("RoundResults", ["count", "elected", "eliminated"])
+
+class PyStvError(Exception):
+    """Error in PyStv."""
+
+
+RoundResult = collections.namedtuple(
+    "RoundResults", ["count", "elected", "eliminated", "transfers"]
+)
 
 
 def run_stv(ballots, votes, num_seats):
@@ -15,8 +23,8 @@ def run_stv(ballots, votes, num_seats):
     weights = np.zeros_like(ballots, float)
     weights[:, 0] = 1
 
+    # TODO: Need to add 1 if it evenly divides
     votes_needed = int(np.ceil(np.sum(votes) / (num_seats + 1)))
-    # votes_needed = int(np.sum(votes) / (num_seats + 1))
 
     status = np.zeros(num_slots, int)
     status[0] = -2
@@ -25,22 +33,33 @@ def run_stv(ballots, votes, num_seats):
     round_info = []
     while np.count_nonzero(status > 0) < num_seats:
         counts = np.bincount(ballots.ravel(), (weights * votes).ravel(), num_slots)
-        counts = np.ma.array(counts, mask=(status != 0))
-        elected = counts >= votes_needed
+        counts_masked = np.ma.array(counts, mask=(status != 0))
+        elected_mask = counts_masked >= votes_needed
 
-        if elected.any():
-            to_remove = np.nonzero(elected)[0]
-            multipliers = [votes_needed / counts[c] for c in to_remove]
+        elected = []
+        eliminated = []
+
+        if np.count_nonzero(status >= 0) == num_seats:
+            to_remove = []
+            elected = np.nonzero(status == 0)[0]
+            multipliers = []
+            status[status == 0] = 1
+        elif elected_mask.any():
+            to_remove = np.nonzero(elected_mask)[0]
+            elected = to_remove
+            multipliers = [votes_needed / counts_masked[c] for c in to_remove]
             status[to_remove] = 1
         else:  # eliminate candidate with fewest votes (random selection if >1)
-            eliminated = np.where(counts == counts.min())[0]
-            to_remove = [np.random.choice(eliminated)]
+            min_counts = np.where(counts_masked == counts_masked.min())[0]
+            to_remove = [np.random.choice(min_counts)]
+            eliminated = to_remove
             multipliers = [0]
             status[to_remove] = -1
 
         # Distribute votes from elected/eliminated candidates to the next active
         # lower ranked candidate.  Elected candidates transfer excess votes, while
         # eliminated candidates transfer all votes.
+        transfers = {}
         for cand, mult in zip(to_remove, multipliers):
             orig = first_nonzero(valid, axis=1)  # Original candidate on each ballot
             valid[ballots == cand] = False
@@ -56,8 +75,20 @@ def run_stv(ballots, votes, num_seats):
             # Votes are transferred from original candidate.
             weights[o_rows, orig[o_rows]] *= mult
 
+            transfers_cand = collections.defaultdict(int)
+            slicer = n_rows, next[n_rows]
+            for c, v in zip(ballots[slicer], (weights * votes)[slicer]):
+                if v:
+                    transfers_cand[c] += v
+
+            if transfers_cand:
+                transfers[cand] = dict(transfers_cand)
+
+        if counts.sum() != votes.sum():
+            raise PyStvError("Final round count total does not equal original votes")
+
         round_info.append(
-            RoundResult(counts, np.nonzero(status == 1)[0], np.nonzero(status == -1)[0])
+            RoundResult(counts.tolist(), list(elected), list(eliminated), transfers)
         )
     return round_info
 
@@ -68,7 +99,10 @@ def first_nonzero(arr, axis, invalid_val=-1):
 
 
 def validate_and_standardize_ballots(ballots):
-    ballots = np.asarray(ballots)
+    ballots = np.array(list(itertools.zip_longest(*ballots, fillvalue=0))).T
+
+    # Add a 0 at the end of every ballot.
+    ballots = np.pad(ballots, ((0, 0), (0, 1)))
 
     if ballots.ndim != 2:
         raise ValueError("Ballot data has wrong dim: %s" % ballots.ndim)
