@@ -1,5 +1,4 @@
 """Tests for `pystv` package."""
-import numpy as np
 import pytest
 from click.testing import CliRunner
 from numpy.testing import assert_equal
@@ -7,6 +6,8 @@ from numpy.testing import assert_equal
 import pystv
 from pystv import cli
 
+from .utils import assert_rr_equal
+
 RR = pystv.RoundResult
 
 
@@ -48,7 +49,7 @@ def test_2cands_1seat():
     results = pystv.run_stv(ballots, votes, num_seats=1)
 
     assert len(results) == 1
-    assert_equal(results[0], RR([0, 1, 2], [2], []))
+    assert_rr_equal(results[0], RR([0, 1, 2], [2], [], {}))
 
 
 def test_2cands_1seat_undervote():
@@ -61,7 +62,26 @@ def test_2cands_1seat_undervote():
     results = pystv.run_stv(ballots, votes, num_seats=1)
 
     assert len(results) == 1
-    assert_equal(results[0], RR([0, 1, 2], [2], []))
+    assert_rr_equal(results[0], RR([0, 1, 2], [2], [], {}))
+
+
+def test_3cands_2seat_undervote():
+    ballots = [
+        [4, 0],
+        [3, 0],
+        [2, 4],
+        [2, 0],
+        [1, 4],
+        [1, 0],
+    ]
+    votes = [3, 3, 1, 1, 1, 1]
+    results = pystv.run_stv(ballots, votes, num_seats=2)
+
+    assert len(results) == 4
+    assert_rr_equal(results[0], RR([0, 2, 2, 3, 3], [], [1], {1: {0: 1, 4: 1}}))
+    assert_rr_equal(results[1], RR([1, 0, 2, 3, 4], [4], [], {}))
+    assert_rr_equal(results[2], RR([1, 0, 2, 3, 4], [], [2], {2: {0: 2}}))
+    assert_rr_equal(results[3], RR([3, 0, 0, 3, 4], [3], [], {}))
 
 
 def test_3cands_2seats_1round():
@@ -73,7 +93,7 @@ def test_3cands_2seats_1round():
     results = pystv.run_stv(ballots, votes, num_seats=2)
 
     assert len(results) == 1
-    assert_equal(results[0], RR([0, 2, 3, 0], [1, 2], []))
+    assert_rr_equal(results[0], RR([0, 2, 3, 0], [1, 2], [], {2: {3: 1}}))
 
 
 def test_3cands_1seat_multiround():
@@ -86,8 +106,8 @@ def test_3cands_1seat_multiround():
     results = pystv.run_stv(ballots, votes, num_seats=1)
 
     assert len(results) == 2
-    assert_equal(results[0], RR([0, 2, 2, 1], [], [3]))
-    assert_equal(results[1], RR([0, 3, 2, 0], [1], [3]))
+    assert_rr_equal(results[0], RR([0, 2, 2, 1], [], [3], {3: {1: 1}}))
+    assert_rr_equal(results[1], RR([0, 3, 2, 0], [1], [], {}))
 
 
 def test_3cands_2seats_multiround():
@@ -100,13 +120,7 @@ def test_3cands_2seats_multiround():
     votes = [2, 4, 1, 2]
     results = pystv.run_stv(ballots, votes, num_seats=2)
     assert len(results) == 1
-    assert_equal(results[0], RR([0, 2, 4, 3], [2, 3], []))
-
-
-#    assert len(results) == 3
-#    assert_equal(results[0], RR([0, 2, 4, 3], [2], []))
-#    assert_equal(results[1], RR([0, 2, 4, 3], [2], [1]))
-#    assert_equal(results[2], RR([0, 0, 4, 5], [2, 3], [1]))
+    assert_rr_equal(results[0], RR([0, 2, 4, 3], [2, 3], [], {2: {1: 1}}))
 
 
 def test_3cands_2seats_multiround_with_adjust():
@@ -119,9 +133,9 @@ def test_3cands_2seats_multiround_with_adjust():
     votes = [2, 5, 1, 2]
     results = pystv.run_stv(ballots, votes, num_seats=2)
     assert len(results) == 3
-    assert_equal(results[0], RR([0, 2, 5, 3], [2], []))
-    assert_equal(results[1], RR([0, 3, 4, 3], [2], [1]))
-    assert_equal(results[2], RR([0, 0, 4, 6], [2, 3], [1]))
+    assert_rr_equal(results[0], RR([0, 2, 5, 3], [2], [], {2: {1: 1}}))
+    assert_rr_equal(results[1], RR([0, 3, 4, 3], [], [1], {1: {3: 3}}))
+    assert_rr_equal(results[2], RR([0, 0, 4, 6], [3], [], {}))
 
 
 def test_fairvote_example():
@@ -132,22 +146,49 @@ def test_fairvote_example():
     ballots, votes = zip(*FAIRVOTE_BALLOTS)
     results = pystv.run_stv(ballots, votes, num_seats=3)
     assert len(results) == 5
-    assert_equal(results[0], RR([0, 2500, 1750, 1300, 1300, 1350, 1000], [1], []))
-    assert_equal(results[1], RR([0, 2300, 1850, 1320, 1300, 1430, 1000], [1], [6]))
-    assert_equal(results[2], RR([0, 2300, 1950, 1900, 1600, 1450, 0], [1], [5, 6]))
-    assert_equal(results[3], RR([0, 2300, 3200, 2050, 1650, 0, 0], [1, 2], [5, 6]))
-    assert_equal(
-        results[4], RR([196.875, 2300, 2300, 2500, 1740, 0, 0], [1, 2, 3], [5, 6])
+    assert_rr_equal(
+        results[0],
+        RR(
+            [0, 2500, 1750, 1300, 1300, 1350, 1000],
+            [1],
+            [],
+            {1: {2: 100, 3: 20, 5: 80}},
+        ),
+    )
+    assert_rr_equal(
+        results[1],
+        RR(
+            [0, 2300, 1850, 1320, 1300, 1430, 1000],
+            [],
+            [6],
+            {6: {2: 100, 3: 580, 4: 300, 5: 20}},
+        ),
+    )
+    assert_rr_equal(
+        results[2],
+        RR(
+            [0, 2300, 1950, 1900, 1600, 1450, 0], [], [5], {5: {2: 1250, 3: 150, 4: 50}}
+        ),
+    )
+    assert_rr_equal(
+        results[3],
+        RR([0, 2300, 3200, 2050, 1650, 0, 0], [2], [], {2: {0: 360, 3: 450, 4: 90}}),
+    )
+    assert_rr_equal(
+        results[4], RR([360, 2300, 2300, 2500, 1740, 0, 0], [3], [], {3: {0: 200}})
     )
 
 
 def test_validate_and_standardize_ballots_ok():
-    ballots = [
-        [1, 0, 0],
-        [1, 2, 0],
-        [1, 2, 3],
-    ]
-    pystv.validate_and_standardize_ballots(ballots)
+    ballots = [[1, 0, 0], [1, 2, 0], [1, 2, 3]]
+    result = pystv.validate_and_standardize_ballots(ballots)
+    assert_equal(result, [[1, 0, 0, 0], [1, 2, 0, 0], [1, 2, 3, 0]])
+
+
+def test_validate_and_standardize_ballots_ragged():
+    ballots = [[1, 0, 0], [1, 2], [1, 2, 3]]
+    result = pystv.validate_and_standardize_ballots(ballots)
+    assert_equal(result, [[1, 0, 0, 0], [1, 2, 0, 0], [1, 2, 3, 0]])
 
 
 def test_validate_and_standardize_ballots_negative():
@@ -156,13 +197,6 @@ def test_validate_and_standardize_ballots_negative():
         pystv.validate_and_standardize_ballots(ballots)
 
 
-@pytest.mark.filterwarnings("ignore", category=np.VisibleDeprecationWarning)
-def test_validate_and_standardize_ballots_ragged():
-    ballots = [[1, 0, 0], [1, 2], [1, 2, 3]]
-    with pytest.raises(ValueError, match=r"Ballot data has wrong dim"):
-        pystv.validate_and_standardize_ballots(ballots)
-
-
 def test_validate_and_standardize_ballots_invalid_ranking():
     ballots = [[1, 0, 0], [0, 1, 0], [1, 2, 0], [0, 0, 1]]
     with pytest.raises(ValueError, match=r"Skipped rankings on ballots: \[1, 3\]"):
